using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CounterInterrupt
{
    public partial class CounterInterrupt : Form
    {
        /// <summary>
        /// Handle to the first board
        /// </summary>
        PCI3016_NET.PCI3016 board;

        /// <summary>
        /// Some internal value. Store the number of interrupt per component
        /// </summary>
        private int counter1InterruptNb = 0;
        private int counter2InterruptNb = 0;
        private int counter3InterruptNb = 0;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public CounterInterrupt()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CounterInterrupt_Load(object sender, EventArgs e)
        {
            int ret = 0;

            // Open the first board of the system
            this.board = PCI3016_NET.PCI3016.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Enable the interruption
            if ((ret = this.board.SetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in SetBoardIntRoutine");
                this.Close();
                return;
            }

            // Stop the components. No check since if nothing is started the function will failed
            for (byte i = 0; i < 3; i++)
            {
                this.board.StopCounterTimerWatchdog(i);
            }

            // Listen to the counter timer watchdog interruption
            this.board.CounterTimerWatchdogEventReceived += new PCI3016_NET.PCI3016.CounterTimerWatchdogEventHandler(board_CounterTimerWatchdogEventReceived);
        }

        /// <summary>
        /// Counter timer watchdog interruption
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_CounterTimerWatchdogEventReceived(object source, PCI3016_NET.CounterTimerWatchdogEventArgs args)
        {
            if (args.InterruptMask == 0)
            {
                // Counter timer 1
                this.counter1InterruptNb++;
            }
            else if (args.InterruptMask == 1)
            {
                // Counter timer 2
                this.counter2InterruptNb++;
            }
            else if (args.InterruptMask == 2)
            {
                // Counter watchdog 3
                this.counter3InterruptNb++;
            }
        }

        /// <summary>
        /// Click on the start button for the counter 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter1Start_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Initialize the component as a counter
            if ( ( ret = this.board.InitAndStartCounterTimerWatchdog(0, // counter,
                0, // counter index
                1, // use interrupt
                100, // reload value
                0, // not used
                0, // output mode: disabled
                0, // not used
                0, // direction up
                2) // count on both level
                ) != 0 )
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartCounterTimerWatchdog");
                return;
            }
            
            // Disable/enable button
            counter1Start.Enabled = false;
            counter1Stop.Enabled = true;
            counter1Trigger.Enabled = true;
            // Start the display timer
            counter1DisplayTimer.Start();
        }

        /// <summary>
        /// Click on the stop button for the counter 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter1Stop_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the counter
            if ((ret = this.board.StopCounterTimerWatchdog(0)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopCounterTimerWatchdog");
                return;
            }
            // Disable/enable button
            counter1Start.Enabled = true;
            counter1Stop.Enabled = false;
            counter1Trigger.Enabled = false;
            // Stop the display timer
            counter1DisplayTimer.Stop();
        }

        /// <summary>
        /// Display timer for counter 1 ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter1DisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            ushort value = 0;
            // Read the value
            if ((ret = this.board.ReadCounterValue(0, ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadCounterValue");
                return;
            }
            counter1Value.Text = value.ToString();
            counter1Interrupt.Text = this.counter1InterruptNb.ToString();
        }

        /// <summary>
        /// Click on the trigger button for the counter 1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter1Trigger_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Read the value
            if ((ret = this.board.TriggerCounterTimerWatchdog(0)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerCounterTimerWatchdog");
                return;
            }
        }

        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CounterInterrupt_FormClosing(object sender, FormClosingEventArgs e)
        {
            int ret = 0;
            // Stop the counter
            for (byte i = 0; i < 3; i++)
            {
                this.board.StopCounterTimerWatchdog(i);
            }
            // Disable the interruption
            if ((ret = this.board.ResetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ResetBoardIntRoutine");
                return;
            }
        }

        /// <summary>
        /// Click on the start button for the counter 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter2Start_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Initialize the component as a counter
            if ((ret = this.board.InitAndStartCounterTimerWatchdog(0, // counter,
                1, // counter index
                1, // use interrupt
                100, // reload value
                0, // not used
                0, // output mode: disabled
                0, // not used
                0, // direction up
                2) // count on both level
                ) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartCounterTimerWatchdog");
                return;
            }

            // Disable/enable button
            counter2Start.Enabled = false;
            counter2Stop.Enabled = true;
            counter2Trigger.Enabled = true;
            // Start the display timer
            counter2DisplayTimer.Start();
        }

        /// <summary>
        /// Display timer for counter 2 ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter2DisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            ushort value = 0;
            // Read the value
            if ((ret = this.board.ReadCounterValue(1, ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadCounterValue");
                return;
            }
            counter2Value.Text = value.ToString();
            counter2Interrupt.Text = this.counter2InterruptNb.ToString();
        }

        /// <summary>
        /// Click on the stop button for the counter 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter2Stop_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the counter
            if ((ret = this.board.StopCounterTimerWatchdog(1)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopCounterTimerWatchdog");
                return;
            }
            // Disable/enable button
            counter2Start.Enabled = true;
            counter2Stop.Enabled = false;
            counter2Trigger.Enabled = false;
            // Stop the display timer
            counter2DisplayTimer.Stop();
        }

        /// <summary>
        /// Click on the trigger button for the counter 2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter2Trigger_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Read the value
            if ((ret = this.board.TriggerCounterTimerWatchdog(1)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerCounterTimerWatchdog");
                return;
            }
        }

        /// <summary>
        /// Click on the start button for the counter 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter3Start_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Initialize the component as a counter
            if ((ret = this.board.InitAndStartCounterTimerWatchdog(0, // counter,
                2, // counter index
                1, // use interrupt
                100, // reload value
                0, // not used
                0, // output mode: disabled
                0, // not used
                0, // direction up
                2) // count on both level
                ) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartCounterTimerWatchdog");
                return;
            }

            // Disable/enable button
            counter3Start.Enabled = false;
            counter3Stop.Enabled = true;
            counter3Trigger.Enabled = true;
            // Start the display timer
            counter3DisplayTimer.Start();
        }

        /// <summary>
        /// Click on the stop button for the counter 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter3Stop_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Stop the counter
            if ((ret = this.board.StopCounterTimerWatchdog(2)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopCounterTimerWatchdog");
                return;
            }
            // Disable/enable button
            counter3Start.Enabled = true;
            counter3Stop.Enabled = false;
            counter3Trigger.Enabled = false;
            // Stop the display timer
            counter3DisplayTimer.Stop();
        }

        /// <summary>
        /// Display timer for counter 3 ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter3DisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            ushort value = 0;
            // Read the value
            if ((ret = this.board.ReadCounterValue(2, ref value)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ReadCounterValue");
                return;
            }
            counter3Value.Text = value.ToString();
            counter3Interrupt.Text = this.counter3InterruptNb.ToString();
        }

        /// <summary>
        /// Click on the trigger button for the counter 3
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void counter3Trigger_Click(object sender, EventArgs e)
        {
            int ret = 0;
            // Read the value
            if ((ret = this.board.TriggerCounterTimerWatchdog(2)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in TriggerCounterTimerWatchdog");
                return;
            }
        }
    }
}